@@ -14,10 +14,10 @@
 /* Initialize BLAS interfacing routines                                     */
 /* ************************************************************************ */
 MYBOOL mustinitBLAS = TRUE;
-#ifdef WIN32
+#if (defined WIN32) || (defined WIN64)
   HINSTANCE hBLAS = NULL;
 #else
-  void      *hBLAS = NULL;
+  void     *hBLAS = NULL;
 #endif
 
 
@@ -30,6 +30,7 @@ BLAS_daxpy_func  *BLAS_daxpy;
 BLAS_dswap_func  *BLAS_dswap;
 BLAS_ddot_func   *BLAS_ddot;
 BLAS_idamax_func *BLAS_idamax;
+BLAS_idamin_func *BLAS_idamin;
 BLAS_dload_func  *BLAS_dload;
 BLAS_dnormi_func *BLAS_dnormi;
 
@@ -61,12 +62,7 @@ MYBOOL load_BLAS(char *libname)
 
 #ifdef LoadableBlasLib
   if(hBLAS != NULL) {
-  #ifdef WIN32
-    FreeLibrary(hBLAS);
-  #else
-    dlclose(hBLAS);
-  #endif
-    hBLAS = NULL;
+    my_FreeLibrary(hBLAS);
   }
 #endif
 
@@ -79,6 +75,7 @@ MYBOOL load_BLAS(char *libname)
     BLAS_dswap = my_dswap;
     BLAS_ddot  = my_ddot;
     BLAS_idamax = my_idamax;
+    BLAS_idamin = my_idamin;
     BLAS_dload = my_dload;
     BLAS_dnormi = my_dnormi;
     if(mustinitBLAS)
@@ -86,58 +83,32 @@ MYBOOL load_BLAS(char *libname)
   }
   else {
 #ifdef LoadableBlasLib
-  #ifdef WIN32
-   /* Get a handle to the Windows DLL module. */
-    hBLAS = LoadLibrary(libname);
-
-   /* If the handle is valid, try to get the function addresses. */
-    result = (MYBOOL) (hBLAS != NULL);
-    if(result) {
-      BLAS_dscal  = (BLAS_dscal_func *)  GetProcAddress(hBLAS, BLAS_prec "scal");
-      BLAS_dcopy  = (BLAS_dcopy_func *)  GetProcAddress(hBLAS, BLAS_prec "copy");
-      BLAS_daxpy  = (BLAS_daxpy_func *)  GetProcAddress(hBLAS, BLAS_prec "axpy");
-      BLAS_dswap  = (BLAS_dswap_func *)  GetProcAddress(hBLAS, BLAS_prec "swap");
-      BLAS_ddot   = (BLAS_ddot_func *)   GetProcAddress(hBLAS, BLAS_prec "dot");
-      BLAS_idamax = (BLAS_idamax_func *) GetProcAddress(hBLAS, "i" BLAS_prec "amax");
-#if 0      
-      BLAS_dload  = (BLAS_dload_func *)  GetProcAddress(hBLAS, BLAS_prec "load");
-      BLAS_dnormi = (BLAS_dnormi_func *) GetProcAddress(hBLAS, BLAS_prec "normi");
-#endif      
-    }
+  #if (defined WIN32) || (defined WIN64)
+    char *blasname = libname;
   #else
    /* First standardize UNIX .SO library name format. */
-    char blasname[260], *ptr;
-
-    strcpy(blasname, libname);
-    if((ptr = strrchr(libname, '/')) == NULL)
-      ptr = libname;
-    else
-      ptr++;
-    blasname[(int) (ptr - libname)] = 0;
-    if(strncmp(ptr, "lib", 3))
-      strcat(blasname, "lib");
-    strcat(blasname, ptr);
-    if(strcmp(blasname + strlen(blasname) - 3, ".so"))
-      strcat(blasname, ".so");
-
-   /* Get a handle to the module. */
-    hBLAS = dlopen(blasname, RTLD_LAZY);
+    char blasname[260];
+    if(!so_stdname(blasname, libname, 260))
+      return( FALSE );
+  #endif
+   /* Get a handle to the Windows DLL module. */
+    hBLAS = my_LoadLibrary(blasname);
 
    /* If the handle is valid, try to get the function addresses. */
     result = (MYBOOL) (hBLAS != NULL);
     if(result) {
-      BLAS_dscal  = (BLAS_dscal_func *)  dlsym(hBLAS, BLAS_prec "scal");
-      BLAS_dcopy  = (BLAS_dcopy_func *)  dlsym(hBLAS, BLAS_prec "copy");
-      BLAS_daxpy  = (BLAS_daxpy_func *)  dlsym(hBLAS, BLAS_prec "axpy");
-      BLAS_dswap  = (BLAS_dswap_func *)  dlsym(hBLAS, BLAS_prec "swap");
-      BLAS_ddot   = (BLAS_ddot_func *)   dlsym(hBLAS, BLAS_prec "dot");
-      BLAS_idamax = (BLAS_idamax_func *) dlsym(hBLAS, "i" BLAS_prec "amax");
+      BLAS_dscal  = (BLAS_dscal_func *)  my_GetProcAddress(hBLAS, BLAS_prec "scal");
+      BLAS_dcopy  = (BLAS_dcopy_func *)  my_GetProcAddress(hBLAS, BLAS_prec "copy");
+      BLAS_daxpy  = (BLAS_daxpy_func *)  my_GetProcAddress(hBLAS, BLAS_prec "axpy");
+      BLAS_dswap  = (BLAS_dswap_func *)  my_GetProcAddress(hBLAS, BLAS_prec "swap");
+      BLAS_ddot   = (BLAS_ddot_func *)   my_GetProcAddress(hBLAS, BLAS_prec "dot");
+      BLAS_idamax = (BLAS_idamax_func *) my_GetProcAddress(hBLAS, "i" BLAS_prec "amax");
+      BLAS_idamin = (BLAS_idamin_func *) my_GetProcAddress(hBLAS, "i" BLAS_prec "amin");
 #if 0      
-      BLAS_dload  = (BLAS_dload_func *)  dlsym(hBLAS, BLAS_prec "load");
-      BLAS_dnormi = (BLAS_dnormi_func *) dlsym(hBLAS, BLAS_prec "normi");
+      BLAS_dload  = (BLAS_dload_func *)  my_GetProcAddress(hBLAS, BLAS_prec "load");
+      BLAS_dnormi = (BLAS_dnormi_func *) my_GetProcAddress(hBLAS, BLAS_prec "normi");
 #endif      
     }
-  #endif
 #endif
     /* Do validation */
     if(!result ||
@@ -147,6 +118,7 @@ MYBOOL load_BLAS(char *libname)
         (BLAS_dswap  == NULL) ||
         (BLAS_ddot   == NULL) ||
         (BLAS_idamax == NULL) ||
+        (BLAS_idamin == NULL) ||
         (BLAS_dload  == NULL) ||
         (BLAS_dnormi == NULL))
       ) {
@@ -179,7 +151,10 @@ void BLAS_CALLMODEL my_daxpy( int *_n, REAL *_da, REAL *dx, int *_incx, REAL *dy
    jack dongarra, linpack, 3/11/78.
    modified 12/3/93, array[1] declarations changed to array[*] */
 
-  int      i, ix, iy, m, mp1;
+  int      i, ix, iy; 
+#ifndef DOFASTMATH
+  int      m, mp1;
+#endif
   register REAL rda;
   REAL     da = *_da;
   int      n = *_n, incx = *_incx, incy = *_incy;
@@ -204,9 +179,8 @@ void BLAS_CALLMODEL my_daxpy( int *_n, REAL *_da, REAL *dx, int *_incx, REAL *dy
     for (i = 1, xptr = dx + ix, yptr = dy + iy;
          i <= n; i++, xptr += incx, yptr += incy)
       (*yptr) += rda*(*xptr);
-    return;
   }
-#endif  
+#else
 
   if (incx==1 && incy==1) goto x20;
 
@@ -235,6 +209,7 @@ x40:
     dy[i + 2]+= rda*dx[i + 2];
     dy[i + 3]+= rda*dx[i + 3];
   }
+#endif  
 }
 
 
@@ -254,18 +229,22 @@ void BLAS_CALLMODEL my_dcopy (int *_n, REAL *dx, int *_incx, REAL *dy, int *_inc
    jack dongarra, linpack, 3/11/78.
    modified 12/3/93, array[1] declarations changed to array[*] */
 
-  int      i, ix, iy, m, mp1;
+  int      i, ix, iy;
+#ifndef DOFASTMATH
+  int      m, mp1;
+#endif
   int      n = *_n, incx = *_incx, incy = *_incy;
 
-  if (n<=0) return;
+  if(n<=0) 
+    return;
 
   dx--;
   dy--;
   ix = 1;
   iy = 1;
-  if (incx<0)
+  if(incx<0)
     ix = (-n+1)*incx + 1;
-  if (incy<0)
+  if(incy<0)
     iy = (-n+1)*incy + 1;
 
 
@@ -276,15 +255,15 @@ void BLAS_CALLMODEL my_dcopy (int *_n, REAL *dx, int *_incx, REAL *dy, int *_inc
     for (i = 1, xptr = dx + ix, yptr = dy + iy;
          i <= n; i++, xptr += incx, yptr += incy)
       (*yptr) = (*xptr);
-    return;
   }
-#endif
+#else
 
-  if (incx==1 && incy==1) goto x20;
+  if(incx==1 && incy==1) 
+    goto x20;
 
 /* code for unequal increments or equal increments not equal to 1 */
 
-  for (i = 1; i<=n; i++) {
+  for(i = 1; i<=n; i++) {
     dy[iy] = dx[ix];
     ix+= incx;
     iy+= incy;
@@ -295,11 +274,6 @@ void BLAS_CALLMODEL my_dcopy (int *_n, REAL *dx, int *_incx, REAL *dy, int *_inc
 
 /* version with fast machine copy logic (requires memory.h or string.h) */
 x20:
-#if defined DOFASTMATH
-  MEMCOPY(&dy[1], &dx[1], n);
-  return;
-#endif
-
   m = n % 7;
   if (m == 0) goto x40;
   for (i = 1; i<=m; i++)
@@ -317,6 +291,7 @@ x40:
      dy[i + 5] = dx[i + 5];
      dy[i + 6] = dx[i + 6];
   }
+#endif
 }
 
 
@@ -346,7 +321,10 @@ void BLAS_CALLMODEL my_dscal (int *_n, REAL *_da, REAL *dx, int *_incx)
      For I = 0 to N-1, replace DX(IX+I*INCX) with  DA * DX(IX+I*INCX),
      where IX = 1 if INCX .GE. 0, else IX = 1+(1-N)*INCX. */
 
-  int      i, ix, m, mp1;
+  int      i;
+#ifndef DOFASTMATH
+  int      m, mp1, ix;
+#endif
   register REAL rda;
   REAL      da = *_da;
   int      n = *_n, incx = *_incx;
@@ -363,9 +341,8 @@ void BLAS_CALLMODEL my_dscal (int *_n, REAL *_da, REAL *dx, int *_incx)
     REAL *xptr;
     for (i = 1, xptr = dx + 1; i <= n; i++, xptr += incx)
       (*xptr) *= rda;
-    return;
   }
-#endif
+#else
 
   if (incx == 1)
     goto x20;
@@ -396,6 +373,7 @@ x40:
     dx[i+3] *= rda;
     dx[i+4] *= rda;
   }
+#endif
 }
 
 
@@ -417,7 +395,10 @@ REAL BLAS_CALLMODEL my_ddot(int *_n, REAL *dx, int *_incx, REAL *dy, int *_incy)
    modified 12/3/93, array[1] declarations changed to array[*] */
 
   register REAL dtemp;
-  int      i, ix, iy, m, mp1;
+  int      i, ix, iy;
+#ifndef DOFASTMATH
+  int      m, mp1;
+#endif
   int      n = *_n, incx = *_incx, incy = *_incy;
 
   dtemp = 0.0;
@@ -441,9 +422,8 @@ REAL BLAS_CALLMODEL my_ddot(int *_n, REAL *dx, int *_incx, REAL *dy, int *_incy)
     for (i = 1, xptr = dx + ix, yptr = dy + iy;
          i <= n; i++, xptr += incx, yptr += incy)
       dtemp+= (*yptr)*(*xptr);
-    return(dtemp);
   }
-#endif
+#else
 
   if (incx==1 && incy==1) goto x20;
 
@@ -474,6 +454,7 @@ x40:
              dx[i + 2]*dy[i + 2] + dx[i + 3]*dy[i + 3] + dx[i + 4]*dy[i + 4];
 
 x60:
+#endif
   return(dtemp);
 }
 
@@ -489,8 +470,12 @@ void dswap( int n, REAL *dx, int incx, REAL *dy, int incy )
 
 void BLAS_CALLMODEL my_dswap( int *_n, REAL *dx, int *_incx, REAL *dy, int *_incy )
 {
-  int   i, ix, iy, m, mp1, ns;
-  REAL  dtemp1, dtemp2, dtemp3;
+  int   i, ix, iy;
+#ifndef DOFASTMATH
+  int   m, mp1, ns;
+  REAL  dtemp2, dtemp3;
+#endif
+  register REAL  dtemp1;
   int   n = *_n, incx = *_incx, incy = *_incy;
 
   if (n <= 0) return;
@@ -514,9 +499,8 @@ void BLAS_CALLMODEL my_dswap( int *_n, REAL *dx, int *_incx, REAL *dy, int *_inc
      (*xptr) = (*yptr);
      (*yptr) = dtemp1;
     }
-    return;
   }
-#endif  
+#else
 
   if (incx == incy) {
     if (incx <= 0) goto x5;
@@ -570,6 +554,7 @@ x60:
      dx[i] = dy[i];
      dy[i] = dtemp1;
   }
+#endif
 
 }
 
@@ -650,6 +635,12 @@ int idamax( int n, REAL *x, int is )
   return ( BLAS_idamax( &n, x, &is ) );
 }
 
+int idamin( int n, REAL *x, int is )
+{
+  x++;
+  return ( BLAS_idamin( &n, x, &is ) );
+}
+
 int BLAS_CALLMODEL my_idamax( int *_n, REAL *x, int *_is )
 {
   register REAL xmax, xtest;
@@ -680,15 +671,53 @@ int BLAS_CALLMODEL my_idamax( int *_n, REAL *x, int *_is )
   xmax = fabs(x[ii]);
   for(i = 2, ii+ = is; i <= n; i++, ii+ = is) {
     xtest = fabs(x[ii]);
-	  if(xtest > xmax) {
+    if(xtest > xmax) {
       xmax = xtest;
-		  imax = i;
+      imax = i;
     }
   }
 #endif  
   return(imax);
 }
 
+int BLAS_CALLMODEL my_idamin( int *_n, REAL *x, int *_is )
+{
+  register REAL xmin, xtest;
+  int    i, imin = 0;
+#if !defined DOFASTMATH
+  int    ii;
+#endif
+  int    n = *_n, is = *_is;
+
+  if((n < 1) || (is <= 0))
+    return(imin);
+  imin = 1;
+  if(n == 1)
+    return(imin);
+
+#if defined DOFASTMATH
+  xmin = fabs(*x);
+  for (i = 2, x += is; i <= n; i++, x += is) {
+    xtest = fabs(*x);
+    if(xtest < xmin) {
+      xmin = xtest;
+      imin = i;
+    }
+  }
+#else
+  x--;
+  ii = 1;
+  xmin = fabs(x[ii]);
+  for(i = 2, ii+ = is; i <= n; i++, ii+ = is) {
+    xtest = fabs(x[ii]);
+    if(xtest < xmin) {
+      xmin = xtest;
+      imin = i;
+    }
+  }
+#endif  
+  return(imin);
+}
 
 /* ************************************************************************ */
 REAL dnormi( int n, REAL *x )
@@ -810,8 +839,8 @@ void ddrand( int n, REAL *x, int incx, int *seeds )
      if (seeds[3] < 0) seeds[3] = seeds[3] + 30323;
 
 	 x[ix]  = ((REAL) seeds[1])/30269.0 +
-             ((REAL) seeds[2])/30307.0 +
-             ((REAL) seeds[3])/30323.0;
+              ((REAL) seeds[2])/30307.0 +
+              ((REAL) seeds[3])/30323.0;
      xix    = (int) x[ix];
 	 x[ix]  = fabs(x[ix] - xix);
    }
