@@ -2,6 +2,7 @@
 #include <time.h>
 #include <signal.h>
 #include <math.h>
+#include <ctype.h>
 #include "lp_lib.h"
 
 #ifdef FORTIFY
@@ -39,7 +40,10 @@ void print_help(char *argv[])
 #if defined PARSER_LP
   printf("-lp\t\tread from LP file (default)\n");
 #endif
-  printf("-mps\t\tread from MPS file in fixed format\n");
+  printf("-mps\t\tread from MPS file, default in fixed format\n");
+  printf("-mps_free\t\tuse free format\n");
+  printf("-mps_ibm\t\tinterprete integers accoring to ibm format\n");
+  printf("-mps_negobjconst\tnegate objective constant\n");
   printf("-fmps\t\tread from MPS file in free format\n");
   printf("-rpar filename\tread parameters from filename.\n");
   printf("-rparopt options\n\t\toptions for parameter file:\n");
@@ -49,6 +53,7 @@ void print_help(char *argv[])
   printf("-rxliopt options\n\t\toptions for xli library.\n");
   printf("-rbas filename\tread basis from filename.\n");
   printf("-gbas filename\tguess basis with variables from filename.\n");
+  printf("-plp\t\tprint model.\n");
   printf("-wlp filename\twrite to LP file\n");
   printf("-wmps filename\twrite to MPS file in fixed format\n");
   printf("-wfmps filename\twrite to MPS file in free format\n");
@@ -217,6 +222,7 @@ void print_help(char *argv[])
   printf("-Da <filename>\tDo a generic readable data dump of key lp_solve model variables\n\t\tafter solve.\n\t\tPrincipally for run difference and debugging purposes\n");
   printf("-i\t\tprint all intermediate valid solutions.\n\t\tCan give you useful solutions even if the total run time\n\t\tis too long\n");
   printf("-ia\t\tprint all intermediate (only non-zero values) valid solutions.\n\t\tCan give you useful solutions even if the total run time\n\t\tis too long\n");
+  printf("-stat\t\tPrint model statistics\n");
   printf("-S <detail>\tPrint solution. If detail omitted, then -S2 is used.\n");
   printf("\t -S0: Print nothing\n");
   printf("\t -S1: Only objective value\n");
@@ -235,8 +241,8 @@ void print_cpu_times(const char *info)
 
   new_time = clock();
   fprintf(stderr, "CPU Time for %s: %gs (%gs total since program start)\n",
-	  info, (new_time - last_time) / (double) CLOCKS_PER_SEC,
-	  new_time / (double) CLOCKS_PER_SEC);
+          info, (new_time - last_time) / (double) CLOCKS_PER_SEC,
+          new_time / (double) CLOCKS_PER_SEC);
   last_time = new_time;
 }
 
@@ -274,8 +280,11 @@ static void __WINAPI LPMessageCB(lprec *lp, void *USERHANDLE, int msg)
     DoReport(lp, "Improved MILP ");
 }
 
-void write_model(lprec *lp, char *wlp, char *wmps, char *wfmps, char *wxli, char *wxlisol, char *wxliname, char *wxlioptions)
+void write_model(lprec *lp, char plp, char *wlp, char *wmps, char *wfmps, char *wxli, char *wxlisol, char *wxliname, char *wxlioptions)
 {
+  if(plp)
+    print_lp(lp);
+
   if(wlp != NULL)
     write_lp(lp, wlp);
 
@@ -316,10 +325,208 @@ static void set_value(int *value, int orvalue)
   *value = orvalue;
 }
 
+#define nstats 5
+
+static void minmax1(REAL value0, REAL factor, REAL *minmax, int *nminmax, int row, int *rowminmax, int col, int *colminmax)
+{
+  int i, n;
+  REAL value;
+
+  if (value0 == 0)
+    return;
+  value = fabs(value0) * factor;
+  for (i = 0; (i < *nminmax) && (value <= fabs(minmax[i]) * factor); i++)
+    if (value == fabs(minmax[i]) * factor)
+      return;
+  if (i >= nstats)
+    return;
+  n = *nminmax;
+  if (n == nstats)
+    n--;
+  memmove(minmax + i + 1, minmax + i, (n - i) * sizeof(*minmax));
+  minmax[i] = value0;
+  if (rowminmax != NULL) {
+    memmove(rowminmax + i + 1, rowminmax + i, (n - i) * sizeof(*rowminmax));
+    rowminmax[i] = row;
+  }
+  if (colminmax != NULL) {
+    memmove(colminmax + i + 1, colminmax + i, (n - i) * sizeof(*colminmax));
+    colminmax[i] = col;
+  }
+  if (*nminmax < nstats)
+    (*nminmax)++;
+}
+
+static void minmax(REAL value, REAL *minima, REAL *maxima, int *nminima, int *nmaxima, int row, int *rowmin, int *rowmax, int col, int *colmin, int *colmax)
+{
+  minmax1(value, -1.0, minima, nminima, row, rowmin, col, colmin);
+  minmax1(value, +1.0, maxima, nmaxima, row, rowmax, col, colmax);
+}
+
+/*
+static void printminmax1(lprec *lp, char *s, REAL *minmax, int nminmax, int *rowminmax, int *colminmax)
+{
+  int i;
+
+  for (i = 0; i < nminmax; i++) {
+    printf(" %s(", s);
+    if (rowminmax != NULL)
+      printf("%s", get_row_name(lp, rowminmax[i]));
+    if ((rowminmax != NULL) && (colminmax != NULL))
+      printf(", ");
+    if (colminmax != NULL)
+      printf("%s", get_col_name(lp, colminmax[i]));
+    printf(") = %.8f\n", minmax[i]);
+  }
+}
+
+static void printminmax(lprec *lp, char *s, REAL *minima, REAL *maxima, int nminima, int nmaxima, int *rowmin, int *rowmax, int *colmin, int *colmax)
+{
+  printf("Minima:\n");
+  printminmax1(lp, s, minima, nminima, rowmin, colmin);
+  printf("Maxima:\n");
+  printminmax1(lp, s, maxima, nmaxima, rowmax, colmax);
+}
+*/
+
+static void printminmax(lprec *lp, char *s, REAL *minima, REAL *maxima, int nminima, int nmaxima, int *rowmin, int *rowmax, int *colmin, int *colmax)
+{
+  int i, nminmax, n;
+
+  nminmax = nminima;
+  if (nmaxima > nminmax)
+    nminmax = nmaxima;
+  for (i = 0; i < nminmax; i++) {
+    n = 0;
+    if (i < nminima) {
+      n += printf("%s(", s);
+      if (rowmin != NULL)
+        n+= printf("%s", get_row_name(lp, rowmin[i]));
+      if ((rowmin != NULL) && (colmin != NULL))
+        n += printf(", ");
+      if (colmin != NULL)
+        n += printf("%s", get_col_name(lp, colmin[i]));
+      n += printf(") = %.8f", minima[i]);
+    }
+    if (n < 40)
+      n = 40 - n;
+    else
+      n = 1;
+    printf("%*.*s", n, n, "");
+
+    if (i < nmaxima) {
+      n += printf("%s(", s);
+      if (rowmax != NULL)
+        n+= printf("%s", get_row_name(lp, rowmax[i]));
+      if ((rowmax != NULL) && (colmax != NULL))
+        n += printf(", ");
+      if (colmax != NULL)
+        n += printf("%s", get_col_name(lp, colmax[i]));
+      n += printf(") = %.8f", maxima[i]);
+    }
+
+    printf("\n");
+  }
+}
+
+static void print_statistics(lprec *lp)
+{
+  REAL *col, *RHSmin, *RHSmax, *OBJmin, *OBJmax, *MATmin, *MATmax;
+  int *nz, ret, m, n, i, j, k, l, nRHSmin = 0, nRHSmax = 0, nOBJmin = 0, nOBJmax = 0, nMATmin = 0, nMATmax = 0, *rowRHSmin, *rowRHSmax, *colOBJmin, *colOBJmax, *rowMATmin, *rowMATmax, *colMATmin, *colMATmax;
+
+  m = get_Nrows(lp);
+  n = get_Ncolumns(lp);
+
+  col = (REAL *) malloc((1 + m) * sizeof(*col));
+  nz = (int *) malloc((1 + m) * sizeof(*RHSmin));
+  RHSmin = (REAL *) malloc(nstats * sizeof(*RHSmin));
+  RHSmax = (REAL *) malloc(nstats * sizeof(*RHSmax));
+  rowRHSmin = (int *) malloc(nstats * sizeof(*RHSmin));
+  rowRHSmax = (int *) malloc(nstats * sizeof(*RHSmax));
+  OBJmin = (REAL *) malloc(nstats * sizeof(*OBJmin));
+  OBJmax = (REAL *) malloc(nstats * sizeof(*OBJmax));
+  colOBJmin = (int *) malloc(nstats * sizeof(*colOBJmin));
+  colOBJmax = (int *) malloc(nstats * sizeof(*colOBJmax));
+  MATmin = (REAL *) malloc(nstats * sizeof(*MATmin));
+  MATmax = (REAL *) malloc(nstats * sizeof(*MATmax));
+  rowMATmin = (int *) malloc(nstats * sizeof(*rowMATmin));
+  rowMATmax = (int *) malloc(nstats * sizeof(*rowMATmax));
+  colMATmin = (int *) malloc(nstats * sizeof(*colMATmin));
+  colMATmax = (int *) malloc(nstats * sizeof(*colMATmax));
+
+/*
+  minmax(2.0, MATmin, MATmax, &nMATmin, &nMATmax, 1, rowMATmin, rowMATmax, 8, colMATmin, colMATmax);
+  minmax(1.0, MATmin, MATmax, &nMATmin, &nMATmax, 2, rowMATmin, rowMATmax, 7, colMATmin, colMATmax);
+  minmax(1.5, MATmin, MATmax, &nMATmin, &nMATmax, 3, rowMATmin, rowMATmax, 6, colMATmin, colMATmax);
+  minmax(3.0, MATmin, MATmax, &nMATmin, &nMATmax, 4, rowMATmin, rowMATmax, 5, colMATmin, colMATmax);
+  minmax(4.0, MATmin, MATmax, &nMATmin, &nMATmax, 5, rowMATmin, rowMATmax, 4, colMATmin, colMATmax);
+  minmax(0.1, MATmin, MATmax, &nMATmin, &nMATmax, 6, rowMATmin, rowMATmax, 3, colMATmin, colMATmax);
+  minmax(5.0, MATmin, MATmax, &nMATmin, &nMATmax, 7, rowMATmin, rowMATmax, 2, colMATmin, colMATmax);
+  minmax(1.4, MATmin, MATmax, &nMATmin, &nMATmax, 8, rowMATmin, rowMATmax, 1, colMATmin, colMATmax);
+
+  printminmax(MATmin, MATmax, nMATmin, nMATmax, rowMATmin, rowMATmax, colMATmin, colMATmax);
+*/
+
+  for (i = 1; i <= m; i++)
+    minmax(get_rh(lp, i), RHSmin, RHSmax, &nRHSmin, &nRHSmax, i, rowRHSmin, rowRHSmax, 0, NULL, NULL);
+
+  for (j = 1; j <= n; j++) {
+    ret = get_columnex(lp, j, col, nz);
+    for (i = 0; i < ret; i++)
+      if (nz[i] == 0)
+        minmax(col[i], OBJmin, OBJmax, &nOBJmin, &nOBJmax, 0, NULL, NULL, j, colOBJmin, colOBJmax);
+      else
+        minmax(col[i], MATmin, MATmax, &nMATmin, &nMATmax, nz[i], rowMATmin, rowMATmax, j, colMATmin, colMATmax);
+  }
+
+  printf("Constraints: %d\n", get_Nrows(lp));
+  printf("Variables  : %d\n", get_Ncolumns(lp));
+  for (j = k = l = 0, i = n; i >= 1; i--) {
+    if (is_int(lp, i))
+      j++;
+    if (is_semicont(lp, i))
+      k++;
+    if (is_SOS_var(lp, i))
+      l++;
+  }
+  printf("Integers   : %d\n", j);
+  printf("Semi-cont  : %d\n", k);
+  printf("SOS        : %d\n", l);
+  k = get_nonzeros(lp);
+  printf("Non-zeros  : %d\tdensity=%f%%\n", k, ((double) k) / (((double) m) * ((double) n)) * 100.0);
+
+  printf("\nAbsolute Ranges:\n\n       Minima                                  Maxima\n");
+  printf("\nMatrix Coeficients:\n");
+  printminmax(lp, "A", MATmin, MATmax, nMATmin, nMATmax, rowMATmin, rowMATmax, colMATmin, colMATmax);
+
+  printf("\nObj. Vector:\n");
+  printminmax(lp, "c", OBJmin, OBJmax, nOBJmin, nOBJmax, NULL, NULL, colOBJmin, colOBJmax);
+
+  printf("\nRHS Vector:\n");
+  printminmax(lp, "b", RHSmin, RHSmax, nRHSmin, nRHSmax, rowRHSmin, rowRHSmax, NULL, NULL);
+
+  free(col);
+  free(nz);
+  free(RHSmin);
+  free(RHSmax);
+  free(rowRHSmin);
+  free(rowRHSmax);
+  free(OBJmin);
+  free(OBJmax);
+  free(colOBJmin);
+  free(colOBJmax);
+  free(MATmin);
+  free(MATmax);
+  free(rowMATmin);
+  free(rowMATmax);
+  free(colMATmin);
+  free(colMATmax);
+}
+
 int main(int argc, char *argv[])
 {
   lprec *lp = NULL;
-  char *filen, *wlp = NULL, *wmps = NULL, *wfmps = NULL;
+  char *filen, *wlp = NULL, *wmps = NULL, *wfmps = NULL, plp = FALSE;
   int i;
   int verbose = IMPORTANT /* CRITICAL */;
   int debug = -1;
@@ -328,6 +535,7 @@ int main(int argc, char *argv[])
   MYBOOL write_model_after = FALSE;
   MYBOOL noint = FALSE;
   int print_sol = -1;
+  MYBOOL print_stats = FALSE;
   int floor_first = -1;
   MYBOOL do_set_bb_depthlimit = FALSE;
   int bb_depthlimit = 0;
@@ -383,6 +591,9 @@ int main(int argc, char *argv[])
   char *wparname = NULL;
   char *wparoptions = NULL;
   char obj_in_basis = -1;
+  char mps_ibm = FALSE;
+  char mps_negobjconst = FALSE;
+  char mps_free = FALSE;
   MYBOOL ok;
 # define SCALINGTHRESHOLD 0.03
 
@@ -408,6 +619,8 @@ int main(int argc, char *argv[])
       print_sol = TRUE;
     else if(strcmp(argv[i], "-ia") == 0)
       print_sol = AUTOMATIC;
+    else if(strcmp(argv[i], "-stat") == 0)
+      print_stats = TRUE;
     else if(strcmp(argv[i], "-nonames") == 0)
       nonames = TRUE;
     else if(strcmp(argv[i], "-norownames") == 0)
@@ -471,9 +684,9 @@ int main(int argc, char *argv[])
     else if((strcmp(argv[i], "-e") == 0) && (i + 1 < argc)) {
       epsint = atof(argv[++i]);
       if((epsint <= 0.0) || (epsint >= 0.5)) {
-	fprintf(stderr, "Invalid tolerance %g; 0 < epsilon < 0.5\n",
-		(double)epsint);
-	EndOfPgr(FORCED_EXIT);
+        fprintf(stderr, "Invalid tolerance %g; 0 < epsilon < 0.5\n",
+                (double)epsint);
+        EndOfPgr(FORCED_EXIT);
       }
     }
     else if((strcmp(argv[i], "-r") == 0) && (i + 1 < argc))
@@ -513,40 +726,40 @@ int main(int argc, char *argv[])
       scaling = SCALE_MEAN;
       if (argv[i][2]) {
         switch (atoi(argv[i] + 2)) {
-	case 0:
-	  scaling = SCALE_NONE;
-	  break;
+        case 0:
+          scaling = SCALE_NONE;
+          break;
         case 1:
-	  set_value(&scalemode1, SCALE_GEOMETRIC);
-	  break;
-	case 2:
-	  set_value(&scalemode1, SCALE_CURTISREID);
-	  break;
-	case 3:
-	  set_value(&scalemode1, SCALE_EXTREME);
-	  break;
+          set_value(&scalemode1, SCALE_GEOMETRIC);
+          break;
+        case 2:
+          set_value(&scalemode1, SCALE_CURTISREID);
+          break;
+        case 3:
+          set_value(&scalemode1, SCALE_EXTREME);
+          break;
         case 4:
-	  set_value(&scalemode1, SCALE_MEAN);
-	  break;
-	case 5:
-	  set_value(&scalemode1, SCALE_MEAN | SCALE_LOGARITHMIC);
-	  break;
+          set_value(&scalemode1, SCALE_MEAN);
+          break;
+        case 5:
+          set_value(&scalemode1, SCALE_MEAN | SCALE_LOGARITHMIC);
+          break;
         case 6:
-	  set_value(&scalemode1, SCALE_RANGE);
-	  break;
-	case 7:
-	  set_value(&scalemode1, SCALE_MEAN | SCALE_QUADRATIC);
-	  break;
+          set_value(&scalemode1, SCALE_RANGE);
+          break;
+        case 7:
+          set_value(&scalemode1, SCALE_MEAN | SCALE_QUADRATIC);
+          break;
         }
       }
       else
         set_value(&scalemode1, SCALE_MEAN);
       if((i + 1 < argc) && (isNum(argv[i + 1])))
-	scaleloop = atoi(argv[++i]);
+        scaleloop = atoi(argv[++i]);
     }
     else if(strncmp(argv[i], "-C", 2) == 0)
       crashmode = atoi(argv[i] + 2);
-	else if((strcmp(argv[i],"-gbas") == 0) && (i + 1 < argc))
+    else if((strcmp(argv[i],"-gbas") == 0) && (i + 1 < argc))
       guessbasis = argv[++i];
     else if(strcmp(argv[i], "-t") == 0)
       tracing = TRUE;
@@ -588,7 +801,7 @@ int main(int argc, char *argv[])
       if (argv[i][4])
         set_value(&pivoting1, atoi(argv[i] + 4));
       else
-	set_value(&pivoting1, PRICER_DEVEX | PRICE_ADAPTIVE);
+    set_value(&pivoting1, PRICER_DEVEX | PRICE_ADAPTIVE);
     }
 #if defined PARSER_LP
     else if(strcmp(argv[i],"-lp") == 0)
@@ -596,8 +809,16 @@ int main(int argc, char *argv[])
 #endif
     else if((strcmp(argv[i],"-wlp") == 0) && (i + 1 < argc))
       wlp = argv[++i];
+    else if(strcmp(argv[i],"-plp") == 0)
+      plp = TRUE;
     else if(strcmp(argv[i],"-mps") == 0)
       filetype = filetypeMPS;
+    else if(strcmp(argv[i],"-mps_ibm") == 0)
+      mps_ibm = TRUE;
+    else if(strcmp(argv[i],"-mps_negobjconst") == 0)
+      mps_negobjconst = TRUE;
+    else if(strcmp(argv[i],"-mps_free") == 0)
+      mps_free = TRUE;
     else if(strcmp(argv[i],"-fmps") == 0)
       filetype = filetypeFREEMPS;
     else if((strcmp(argv[i],"-wmps") == 0) && (i + 1 < argc))
@@ -630,9 +851,9 @@ int main(int argc, char *argv[])
       or_value(&anti_degen2, ANTIDEGEN_BOUNDFLIP);
     else if(strcmp(argv[i],"-time") == 0) {
       if(clock() == -1)
-	fprintf(stderr, "CPU times not available on this machine\n");
+        fprintf(stderr, "CPU times not available on this machine\n");
       else
-	print_timing = TRUE;
+        print_timing = TRUE;
     }
     else if((strcmp(argv[i],"-bfp") == 0) && (i + 1 < argc))
       bfp = argv[++i];
@@ -737,19 +958,19 @@ int main(int argc, char *argv[])
       i++;
     else if((strcmp(argv[i],"-wparopt") == 0) && (i + 1 < argc))
       i++;
-	else if(strcmp(argv[i],"-o0") == 0)
-	  obj_in_basis = FALSE;
-	else if(strcmp(argv[i],"-o1") == 0)
-	  obj_in_basis = TRUE;
+    else if(strcmp(argv[i],"-o0") == 0)
+      obj_in_basis = FALSE;
+    else if(strcmp(argv[i],"-o1") == 0)
+      obj_in_basis = TRUE;
     else if(fpin == stdin) {
       filen = argv[i];
       if(*filen == '<')
         filen++;
       if((fpin = fopen(filen, "r")) == NULL) {
-	print_help(argv);
-	fprintf(stderr,"\nError, Unable to open input file '%s'\n",
-		argv[i]);
-	EndOfPgr(FORCED_EXIT);
+        print_help(argv);
+        fprintf(stderr,"\nError, Unable to open input file '%s'\n",
+                argv[i]);
+        EndOfPgr(FORCED_EXIT);
       }
     }
     else {
@@ -757,7 +978,7 @@ int main(int argc, char *argv[])
       if(*filen != '>') {
         print_help(argv);
         fprintf(stderr, "\nError, Unrecognized command line argument '%s'\n",
-		argv[i]);
+                argv[i]);
         EndOfPgr(FORCED_EXIT);
       }
     }
@@ -765,21 +986,27 @@ int main(int argc, char *argv[])
 
   signal(SIGABRT,/* (void (*) OF((int))) */ SIGABRT_func);
 
-  switch(filetype) {
-#if defined PARSER_LP
-  case filetypeLP:
-    lp = read_lp(fpin, verbose, NULL);
-    break;
-#endif
-  case filetypeMPS:
-    lp = read_mps(fpin, verbose);
-    break;
-  case filetypeFREEMPS:
-    lp = read_freemps(fpin, verbose);
-    break;
-  case filetypeXLI:
-    lp = read_XLI(rxliname, rxli, rxlidata, rxlioptions, verbose);
-    break;
+  if ((filetype != filetypeXLI) && (fpin == NULL)) {
+    lp = NULL;
+    fprintf(stderr, "Cannot combine -rxli option with -lp, -mps, -fmps.\n");
+  }
+  else {
+    switch(filetype) {
+  #if defined PARSER_LP
+    case filetypeLP:
+      lp = read_lp(fpin, verbose, NULL);
+      break;
+  #endif
+    case filetypeMPS:
+      lp = read_mps(fpin, verbose | (mps_free ? MPS_FREE : 0) | (mps_ibm ? MPS_IBM : 0) | (mps_negobjconst ? MPS_NEGOBJCONST : 0));
+      break;
+    case filetypeFREEMPS:
+      lp = read_freemps(fpin, verbose | (mps_ibm ? MPS_IBM : 0) | (mps_negobjconst ? MPS_NEGOBJCONST : 0));
+      break;
+    case filetypeXLI:
+      lp = read_XLI(rxliname, rxli, rxlidata, rxlioptions, verbose);
+      break;
+    }
   }
 
   if((fpin != NULL) && (fpin != stdin))
@@ -796,11 +1023,11 @@ int main(int argc, char *argv[])
   for(i = 1; i < argc; i++) {
     if((strcmp(argv[i],"-rpar") == 0) && (i + 1 < argc)) {
       if(rparname != NULL) {
-	if(!read_params(lp, rparname, rparoptions)) {
-	  fprintf(stderr, "Unable to read parameter file (%s)\n", rparname);
-	  delete_lp(lp);
-	  EndOfPgr(FORCED_EXIT);
-	}
+        if(!read_params(lp, rparname, rparoptions)) {
+          fprintf(stderr, "Unable to read parameter file (%s)\n", rparname);
+          delete_lp(lp);
+          EndOfPgr(FORCED_EXIT);
+        }
       }
       rparname = argv[++i];
     }
@@ -838,8 +1065,8 @@ int main(int argc, char *argv[])
     for(i = get_Ncolumns(lp); i >= 1; i--) {
       if(is_SOS_var(lp, i)) {
         fprintf(stderr, "Unable to remove integer conditions because there is at least one SOS constraint\n");
-	delete_lp(lp);
-	EndOfPgr(FORCED_EXIT);
+        delete_lp(lp);
+        EndOfPgr(FORCED_EXIT);
       }
       set_semicont(lp, i, FALSE);
       set_int(lp, i, FALSE);
@@ -847,7 +1074,10 @@ int main(int argc, char *argv[])
   }
 
   if(!write_model_after)
-    write_model(lp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+    write_model(lp, plp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+
+  if(print_stats)
+    print_statistics(lp);
 
   if(parse_only) {
     if(!write_model_after) {
@@ -961,51 +1191,51 @@ int main(int argc, char *argv[])
 
   if(guessbasis != NULL) {
     REAL *guessvector, a;
-	int *basisvector;
-	int Nrows = get_Nrows(lp);
-	int Ncolumns = get_Ncolumns(lp);
-	int col;
-	char buf[50], *ptr;
-	FILE *fp;
+    int *basisvector;
+    int Nrows = get_Nrows(lp);
+    int Ncolumns = get_Ncolumns(lp);
+    int col;
+    char buf[50], *ptr;
+    FILE *fp;
 
-	if ((fp = fopen(guessbasis, "r")) != NULL) {
-	  guessvector = (REAL *) calloc(1+Ncolumns, sizeof(*guessvector));
-	  basisvector = (int *) malloc((1+Nrows+Ncolumns)*sizeof(*basisvector));
-	  if ((guessvector != NULL) && (basisvector != NULL)) {
-	    while ((!feof(fp)) && (fgets(buf, sizeof(buf), fp) != NULL)) {
-  		  ptr = strrchr(buf, ':');
-		  if (ptr == NULL) {
-			printf("Mallformed line: %s\n", buf);
-		  }
-		  else {
-		    a = atof(ptr + 1);
-		    while ((ptr > buf) && (isspace(ptr[-1])))
-			  ptr--;
-		    *ptr = 0;
-			col = get_nameindex(lp, buf, FALSE);
-			if (col < 1)
-			  printf("guess_basis: Unknown variable name %s\n", buf);
-			else
-		      guessvector[col] = a;
-		  }
-		}
-	    if (guess_basis(lp, guessvector, basisvector)) {
-	      if (!set_basis(lp, basisvector, TRUE))
-		    printf("Unable to set guessed basis.\n");
-	    }
-	    else
-	      printf("Unable to guess basis from provided variables.\n");
-	  }
-	  else
-	    printf("guess_basis: Out of memory.\n");
-	  if (basisvector != NULL)
-	    free(basisvector);
-	  if (guessvector != NULL)
-	    free(guessvector);
-	  fclose(fp);
-	}
-	else
-	  printf("Unable to open file %s\n", guessbasis);
+    if ((fp = fopen(guessbasis, "r")) != NULL) {
+      guessvector = (REAL *) calloc(1+Ncolumns, sizeof(*guessvector));
+      basisvector = (int *) malloc((1+Nrows+Ncolumns)*sizeof(*basisvector));
+      if ((guessvector != NULL) && (basisvector != NULL)) {
+        while ((!feof(fp)) && (fgets(buf, sizeof(buf), fp) != NULL)) {
+          ptr = strrchr(buf, ':');
+          if (ptr == NULL) {
+            printf("Mallformed line: %s\n", buf);
+          }
+          else {
+            a = atof(ptr + 1);
+            while ((ptr > buf) && (isspace(ptr[-1])))
+              ptr--;
+            *ptr = 0;
+            col = get_nameindex(lp, buf, FALSE);
+            if (col < 1)
+              printf("guess_basis: Unknown variable name %s\n", buf);
+            else
+              guessvector[col] = a;
+          }
+        }
+        if (guess_basis(lp, guessvector, basisvector)) {
+          if (!set_basis(lp, basisvector, TRUE))
+            printf("Unable to set guessed basis.\n");
+        }
+        else
+          printf("Unable to guess basis from provided variables.\n");
+      }
+      else
+        printf("guess_basis: Out of memory.\n");
+      if (basisvector != NULL)
+        free(basisvector);
+      if (guessvector != NULL)
+        free(guessvector);
+      fclose(fp);
+    }
+    else
+      printf("Unable to open file %s\n", guessbasis);
   }
 
   if(rbasname != NULL)
@@ -1022,9 +1252,9 @@ int main(int argc, char *argv[])
       fprintf(stderr, "Unable to write basis file.\n");
 
   if(write_model_after)
-    write_model(lp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+    write_model(lp, plp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
 
-  write_model(lp, NULL, NULL, NULL, NULL, wxlisol, wxliname, wxlisoloptions);
+  write_model(lp, FALSE, NULL, NULL, NULL, NULL, wxlisol, wxliname, wxlisoloptions);
 
   if(PRINT_SOLUTION >= 6)
     print_scales(lp);
@@ -1078,8 +1308,8 @@ int main(int argc, char *argv[])
 
     if(tracing)
       fprintf(stderr,
-      "Branch & Bound depth: %d\nNodes processed: %.0f\nSimplex pivots: %.0f\nNumber of equal solutions: %d\n",
-	      get_max_level(lp), (REAL) get_total_nodes(lp), (REAL) get_total_iter(lp), get_solutioncount(lp));
+              "Branch & Bound depth: %d\nNodes processed: %.0f\nSimplex pivots: %.0f\nNumber of equal solutions: %d\n",
+              get_max_level(lp), (REAL) get_total_nodes(lp), (REAL) get_total_iter(lp), get_solutioncount(lp));
     break;
   case NOMEMORY:
     if (PRINT_SOLUTION >= 1)
